/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "kitaconfig.h"

#include <kconfig.h>

KitaConfig* KitaConfig::instance = 0;

KitaConfig* KitaConfig::getInstance()
{
    if ( !instance ) {
        instance = new KitaConfig();
    }
    return instance;
}

KitaConfig::KitaConfig()
        : m_threadColor( "black" ), m_threadBackgroundColor( "white" ),
        m_popupColor( "black" ), m_popupBackgroundColor( "yellow" ),
        m_showMailAddress( false ), m_MarkTime( 24 ), m_alwaysUseTab( false ),
        m_viewMode( View_2pane ), m_sortOrder( Order_Mark ), m_showAA( false ),
        m_showNum( 100 )
{}

KitaConfig::~KitaConfig()
{}

void KitaConfig::writeConfig( KConfig* config )
{
    config->setGroup( "Global" );
    config->writeEntry( "Font", KitaConfig::font() );
    config->writeEntry( "ThreadFont", KitaConfig::threadFont() );
    config->writeEntry( "ShowMailAddress", KitaConfig::showMailAddress() );
    config->writeEntry( "MarkTime", KitaConfig::MarkTime() );
    config->writeEntry( "SortOrder", KitaConfig::sortOrder() );
    config->writeEntry( "AlwaysUseTab", KitaConfig::alwaysUseTab() );
    config->writeEntry( "ShowAA", KitaConfig::showAA() );
    config->writeEntry( "ShowNum", KitaConfig::showNum() );
    config->writeEntry( "UsePart", KitaConfig::usePart() );
    config->writeEntry( "PartMimeList", KitaConfig::partMimeList() );

    config->setGroup( "Color" );
    config->writeEntry( "Popup", KitaConfig::popupColor() );
    config->writeEntry( "PopupBackground", KitaConfig::popupBackgroundColor() );
    
    config->setGroup( "Account" );
    config->writeEntry( "UserID", KitaConfig::userID() );
    config->writeEntry( "Password", encryptStr( KitaConfig::password() ) );
}

void KitaConfig::readConfig( KConfig* config )
{
    config->setGroup( "Global" );
    KitaConfig::setFont( config->readFontEntry( "Font" ) );
    KitaConfig::setThreadFont( config->readFontEntry( "ThreadFont" ) );
    KitaConfig::setShowMailAddresss( config->readBoolEntry( "ShowMailAddress" ) );
    KitaConfig::setMarkTime( config->readUnsignedNumEntry( "MarkTime" , 24 ) );
    KitaConfig::setViewMode( config->readNumEntry( "ViewMode", KitaConfig::View_2pane ) );
    KitaConfig::setSortOrder( config->readNumEntry( "SortOrder", KitaConfig::Order_Mark ) );
    KitaConfig::setAlwaysUseTab( config->readBoolEntry( "AlwaysUseTab", false ) );
    KitaConfig::setShowAA( config->readBoolEntry( "ShowAA", false ) );
    KitaConfig::setShowNum( config->readNumEntry( "ShowNum", 100 ) );
    KitaConfig::setUsePart( config->readBoolEntry( "UsePart", true ) );
    if ( config->hasKey( "PartMimeList" ) )
        KitaConfig::setPartMimeList( config->readListEntry( "PartMimeList" ) );
    else {
        // set partMimeList default
        QStringList list;
        list << "image/gif" << "image/jpeg" << "image/png" << "image/x-bmp";
        KitaConfig::setPartMimeList( list );
    }

    setFont( KitaConfig::font() );

    config->setGroup( "Color" );

    QColor popupColor = config->readColorEntry( "Popup" );
    if ( popupColor.isValid() )
    {
        KitaConfig::setPopupColor( popupColor );
    }

    QColor popupBackgroundColor = config->readColorEntry( "PopupBackground" );
    if ( popupColor.isValid() )
    {
        KitaConfig::setPopupBackgroundColor( popupBackgroundColor );
    }
    
    config->setGroup( "Account" );
    KitaConfig::setUserID( config->readEntry( "UserID" ) );
    KitaConfig::setPassword( decryptStr( config->readEntry( "Password" ) ) );
}

/*
 * encryptStr and decryptStr are copied from kdepim/kmail/kmaccount.{h,cpp}
 */
QString KitaConfig::encryptStr( const QString& aStr )
{
    QString result;
    for (uint i = 0; i < aStr.length(); i++)
        result += (aStr[i].unicode() < 0x20) ? aStr[i] :
                  QChar(0x1001F - aStr[i].unicode());
    return result;
}

QString KitaConfig::decryptStr( const QString& aStr )
{
    return encryptStr( aStr );
}
